/* titel: baum.c; Binaerbaum als Beispiel zu selbstreferentiellen Strukturen  */
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <locale.h>
#define MAX_WORT      40
char *progname;
struct  node {
            char *wort;
            int  zahl;
            struct node *links;
            struct node *rechts;
};
typedef struct node knoten;
/*  main liest aus der Standard-Eingabe die Woerter und gibt sie
lexikographisch sortiert mit Haeufigkeitsangabe auf die
Standard-Ausgabe aus. */
int dcount = 0;
main() {   
    knoten *unterbaum, *suche();
    int wcount=0;
    char   wort [ MAX_WORT ] ;
    int    c;
    setlocale(LC_ALL, "deutsch");
    unterbaum = NULL;
    while ( ( c = lieswort( wort, MAX_WORT) ) != EOF )
      if (c == 'a') {
	unterbaum = suche( unterbaum, wort );
	wcount++;
      }
    printf("%d Woerter gelesen, davon paarweise verschieden: %d\n",wcount, dcount);
    inorder( unterbaum );
}
/*  lieswort liest aus einer Datei ein Wort der Laenge < lim in den
Speicher, auf den w zeigt, und gibt 'a' zurueck, falls es wirklich
Alphazeichen gelesen hat, jedoch EOF im Fall zu grosser
Wortlaenge. Falls Nicht-Alpha-Zeichen gelesen werden, werden diese
einzeln zurueckgegeben. */
int lieswort(char *w, int lim) {   
    int c;
    while ( !isalpha( c = getchar() ) ) 
      if ( c == EOF ) return c;
    *w++ = c; lim--;   /* Alpha-Zeichen gesehen, suche nach weiteren:  */
    while ( isalpha( c = getchar() )  && lim > 0 ) {    
      *w++ = c; lim--;   
    }
    if   (lim == 0) {     
      printf("Error: Wort zu lang "); return EOF; 
    }
    else *w = 0;
    return 'a'; 
}
/*   suche  durchsucht den Baum auf das Vorkommen des Wortes w,
     traegt es gegebenenfalls lexikographisch richtig ein bzw. erhoeht
     dessen Zahl und gibt den Pointer auf seinen Knoten zurueck.
     Die c-Funktion strcmp (resp. strcoll) vergleicht zwei 
     Zeichenketten lexikographisch
     mit Rueckgabe einer negativen Zahl, 0 oder einer positiven Zahl. */
knoten *suche(knoten *p, char *w) {    
     knoten *p_knoten();
     char *merke_wort();
     int  cond;
     if   (p == NULL) {
          p = p_knoten();
          p->wort = merke_wort(w);
          p->zahl = 1;
          p->links = p->rechts = NULL;
	  dcount++;
     }
     else if   ((cond = strcoll(w, p->wort)) == 0)
               p->zahl++;
          else if   (cond < 0)
                    p->links = suche(p->links, w);
               else p->rechts = suche(p->rechts, w);
     return p ;
}
/* p_knoten gibt einen Zeiger auf freien Speicherplatz fuer eine
Knoten-Variable zurueck. */
knoten *p_knoten(void) {    
     return (knoten *)malloc( sizeof( knoten ) ) ;
}
/*   inorder  gibt den Unterbaum, auf den p zeigt, in Inorder aus.
     (rekursive Version!)                  */
inorder(knoten *p) {    
     if (p != NULL){  
       inorder(p->links);
       printf("%4d %s\n", p->zahl, p->wort);
       inorder(p->rechts);
     }
}
/*  merke_wort speichert das Wort, auf dessen ersten Charakter s
zeigt, und gibt einen Zeiger auf dessen neue Lokation an.  die
c-Funktionen strlen, strcpy geben die Laenge einer Zeichenkette
zurueck beziehungsweise kopieren diese. */
char *merke_wort(char *s) {   
    char *p;
    if ( ( p = (char *)malloc( strlen(s)+1 ) ) != NULL )
       strcpy(p, s);
    return p ;
}
