// btree_qsort.cc : Binaerbaum + Quicksort

#include <ctype.h>
#include <iostream.h>
#include <locale.h>
#include "quicksort.h"
#include "string-class.h"

class pair {
public:
  string s; int n;
  friend int operator<(pair& x, pair& y) {
    if (x.n == y.n) return (x.s < y.s);
    return (x.n < y.n);
  }
  friend int operator>(pair& x, pair& y) {
    if (x.n == y.n) return (x.s > y.s);
    return (x.n > y.n);
  }

};

template<class T>
class tree {
  struct node{
    T e; int c; int s; node *l, *r;   
    // Element, Zaehler, Groesse, linker/rechter Teilbaum
    node() { c = s = 1; l = NULL; r = NULL; }
  } *n;
  friend int size (node *n)  { return ( n ? n->s : 0 );  }
  void consist(node *n) { 
    if (n) { n->s = 1 + size(n->l) + size(n->r); }
  }

public:
  tree() { n = NULL; }        // Konstruktor : fuer Deklarationen
  tree(node *nn) { n = nn;}   // Konstruktor : fuer Initialisierungen
  friend int size(tree t) { return size(t.n); }
  node* insert(const T &e) {  // Member-Fumktion : 
    if (n == NULL) { 
      n = new node;           // liefert und initialisiert struct node n
      n->e = e;               // Datenelement   
    }
    else if (e == n->e) { n->c++; } // e vorhanden, erhoehe Zaehler
    else if (e <  n->e)             // ordne lexikographisch ein :
      n->l = tree(n->l).insert(e);
    else 
      n->r = tree(n->r).insert(e); 
    consist(n);
    return this->n;
  }
  friend ostream& inorder(ostream& os, tree t) {
    if (t.n) {  
      inorder(os, t.n->l);
      os << t.n->e << '(' << t.n->c << ")\n" ; 
      inorder(os, t.n->r);
    }
    return os;
  }
  friend pair* inorder(pair* p, tree t) {
    if (t.n) {  
      p = inorder(p, t.n->l);
      p->s = t.n->e; p->n = t.n->c; p++;
      p = inorder(p, t.n->r); 
    }
    return p;
  }
};

bool getword(istream& is, string& s) {
  if ( ! (is >> s) ) return false ;
  char *p, *q;
  p = q = &s[0]; 
  while (*p) {       
    if ( isalpha(*p) ) // Nur alpha-Zeichen werden beruecksichtigt
      *q++ = *p;       // Kopieren von Zeichen 
    p++;
  }
  *q = '\0';
  return true;
}

int main() {
  string s;
  tree<string> tr;              // Deklariere Baum aus Strings 
  setlocale(LC_ALL, "deutsch"); // cf. "man setlocale", "man locale"
  while ( getword(cin, s) ) {
    tr.insert(s);
  }

  pair *p;
  cout << size(tr) << "\n";
  p = new pair[ size(tr) ];

  cout << "Inorder arbeitet:\n";
  inorder(p, tr);     // Ausgabe der sortierten Woerter in Array
  cout << "Quicksort arbeitet:\n";
  quicksort(p,size(tr));
  for (int i = 0 ; i < size(tr) ; i++) {
    cout << p[i].n << " " << p[i].s << "\n";
  }
}
